@@ -16,6 +16,7 @@
 #include <KLocalizedString>
 #include <KTextEditor/Message>
 #include <KTextEditor/View>
+#include <utility>
 
 #include <drawing_utils.h>
 
@@ -87,9 +88,9 @@ public:
 };
 
 BranchesDialog::BranchesDialog(QWidget *window, QString projectPath)
-    : HUDDialog(nullptr, window)
+    : HUDDialog(window)
     , m_model(new BranchesDialogModel(this))
-    , m_projectPath(projectPath)
+    , m_projectPath(std::move(projectPath))
 {
     setModel(m_model, FilterType::ScoredFuzzy, 0, Qt::DisplayRole, BranchesDialogModel::FuzzyScore);
     setDelegate(new StyleDelegate(this));
@@ -103,21 +104,8 @@ void BranchesDialog::openDialog(GitUtils::RefType r)
     m_model->refresh(branches);
 
     reselectFirst();
-    exec();
-}
-
-void BranchesDialog::slotReturnPressed(const QModelIndex &index)
-{
-    if (index.isValid()) {
-        const auto branch = index.data().toString();
-        const auto itemType = (BranchesDialogModel::ItemType)index.data(BranchesDialogModel::ItemTypeRole).toInt();
-        Q_ASSERT(itemType == BranchesDialogModel::BranchItem);
-
-        m_branch = branch;
-    }
-
-    clearLineEdit();
-    hide();
+    raise();
+    show();
 }
 
 void BranchesDialog::sendMessage(const QString &plainText, bool warn)
