@@ -64,6 +64,7 @@
 
 #include <QApplication>
 #include <QCommandLineParser>
+#include <QElapsedTimer>
 #include <QFileInfo>
 #include <QFileOpenEvent>
 #include <QJsonArray>
@@ -430,7 +431,7 @@ bool KateApp::init()
     // the session will only get saved if Kate is properly closed and not if it gets killed.
     // Do it delayed, don't block the startup
     QTimer::singleShot(100, this, [this] {
-        sessionManager()->saveActiveSession(true);
+        sessionManager()->saveActiveSession(true, /*isAutoSave=*/true);
     });
 
     return true;
@@ -508,22 +509,24 @@ bool KateApp::startupKate()
     KTextEditor::Document *doc = nullptr;
     const QString codec_name = m_args.isSet(QStringLiteral("encoding")) ? m_args.value(QStringLiteral("encoding")) : QString();
 
-    const auto args = m_args.positionalArguments();
+    auto args = m_args.positionalArguments();
 
-    for (const auto &positionalArgument : args) {
-        UrlInfo info(positionalArgument);
+    // support +xyz line number as first argument
+    KTextEditor::Cursor firstLineNumberArgument = UrlInfo::parseLineNumberArgumentAndRemoveIt(args);
 
+    for (UrlInfo info : std::as_const(args)) {
         // this file is no local dir, open it, else warn
-        bool noDir = !info.url.isLocalFile()
-            || KNetworkMounts::self()->isOptionEnabledForPath(info.url.toLocalFile(), KNetworkMounts::LowSideEffectsOptimizations)
-            || !QFileInfo(info.url.toLocalFile()).isDir();
-
-        if (noDir) {
+        if (!info.url.isLocalFile() || KNetworkMounts::self()->isOptionEnabledForPath(info.url.toLocalFile(), KNetworkMounts::LowSideEffectsOptimizations)
+            || !QFileInfo(info.url.toLocalFile()).isDir()) {
             if (!info.cursor.isValid()) {
                 if (hasCursorInArgs()) {
                     info.cursor = cursorFromArgs();
                 } else if (info.url.hasQuery()) {
                     info.cursor = cursorFromQueryString(info.url);
+                } else if (firstLineNumberArgument.isValid()) {
+                    // same semantics as e.g. in mcedit, just use for the first file
+                    info.cursor = firstLineNumberArgument;
+                    firstLineNumberArgument = KTextEditor::Cursor::invalid();
                 }
             }
             doc = openDocUrl(info.url, codec_name, tempfileSet, /*activateView=*/false, info.cursor);
@@ -885,7 +888,7 @@ void KateApp::remoteMessageReceived(quint32, QByteArray message)
     }
 
     // try to activate current window
-    activate();
+    activate(jsonMessage.object().value(QLatin1String("activationToken")).toString());
     if (doc && activeMainWindow()) {
         activeMainWindow()->activateView(doc);
     }
@@ -928,25 +931,9 @@ qint64 KateApp::lastActivationChange() const
 void KateApp::activate(const QString &token)
 {
     KateMainWindow *win = activeKateMainWindow();
-    if (!win) {
-        return;
+    if (win) {
+        win->activate(token);
     }
-
-    // like QtSingleApplication
-    win->setWindowState(win->windowState() & ~Qt::WindowMinimized);
-    win->raise();
-    win->activateWindow();
-
-    // try to raise window, see bug 407288
-    if (KWindowSystem::isPlatformX11()) {
-#if HAVE_X11
-        KStartupInfo::setNewStartupId(win->windowHandle(), token.toUtf8());
-#endif
-    } else if (KWindowSystem::isPlatformWayland()) {
-        KWindowSystem::setCurrentXdgActivationToken(token);
-    }
-
-    KWindowSystem::activateWindow(win->windowHandle());
 }
 
 #include "moc_kateapp.cpp"
